# 17 "ml/parsing/lexer.mll"
 
open Lexing
open Misc
open Parser

type error =
  | Illegal_character of char
  | Illegal_escape of string
  | Unterminated_comment
  | Unterminated_string
  | Unterminated_string_in_comment
  | Keyword_as_label of string
  | Literal_overflow of string
  | Unknown_backslash_identifer of string
  | Not_in_annotation
;;

exception Error of error * Location.t;;

let in_annotation = ref false

(* The table of keywords *)

let keyword_table =
  create_hashtable 149 [
    "and", AND;
    "as", AS;
    "assert", ASSERT;
    "begin", BEGIN;
    "class", CLASS;
    "constraint", CONSTRAINT;
    "do", DO;
    "done", DONE;
    "downto", DOWNTO;
    "else", ELSE;
    "end", END;
    "exception", EXCEPTION;
    "external", EXTERNAL;
    "false", FALSE;
    "for", FOR;
    "fun", FUN;
    "function", FUNCTION;
    "functor", FUNCTOR;
    "if", IF;
    "in", IN;
    "include", INCLUDE;
    "inherit", INHERIT;
    "initializer", INITIALIZER;
    "lazy", LAZY;
    "let", LET;
    "match", MATCH;
    "method", METHOD;
    "module", MODULE;
    "mutable", MUTABLE;
    "new", NEW;
    "object", OBJECT;
    "of", OF;
    "open", OPEN;
    "or", OR;
(*  "parser", PARSER; *)
    "private", PRIVATE;
    "rec", REC;
    "sig", SIG;
    "struct", STRUCT;
    "then", THEN;
    "to", TO;
    "true", TRUE;
    "try", TRY;
    "type", TYPE;
    "val", VAL;
    "virtual", VIRTUAL;
    "when", WHEN;
    "while", WHILE;
    "with", WITH;

    "mod", INFIXOP3("mod");
    "land", INFIXOP3("land");
    "lor", INFIXOP3("lor");
    "lxor", INFIXOP3("lxor");
    "lsl", INFIXOP4("lsl");
    "lsr", INFIXOP4("lsr");
    "asr", INFIXOP4("asr")
]

(* The table of annotation-specific keywords *)

let annot_keyword_table =
  create_hashtable 149 [
    "axiom", AXIOM;
    "behavior", BEHAVIOR;
    "ensures", ENSURES;
    "invariant", INVARIANT;
    "logic", LOGIC;
    "predicate", PREDICATE;
    "reads", READS;
    "requires", REQUIRES;
    "variant", VARIANT;
    "\\result", BSRESULT;
    "\\old", BSOLD;
  ]

(* To buffer string literals *)

let initial_string_buffer = String.create 256
let string_buff = ref initial_string_buffer
let string_index = ref 0

let reset_string_buffer () =
  string_buff := initial_string_buffer;
  string_index := 0

let store_string_char c =
  if !string_index >= String.length (!string_buff) then begin
    let new_buff = String.create (String.length (!string_buff) * 2) in
      String.blit (!string_buff) 0 new_buff 0 (String.length (!string_buff));
      string_buff := new_buff
  end;
  String.unsafe_set (!string_buff) (!string_index) c;
  incr string_index

let get_stored_string () =
  let s = String.sub (!string_buff) 0 (!string_index) in
  string_buff := initial_string_buffer;
  s

(* To store the position of the beginning of a string and comment *)
let string_start_loc = ref Location.none;;
let comment_start_loc = ref [];;
let in_comment () = !comment_start_loc <> [];;

(* To translate escape sequences *)

let char_for_backslash = function
  | 'n' -> '\010'
  | 'r' -> '\013'
  | 'b' -> '\008'
  | 't' -> '\009'
  | c   -> c

let char_for_decimal_code lexbuf i =
  let c = 100 * (Char.code(Lexing.lexeme_char lexbuf i) - 48) +
           10 * (Char.code(Lexing.lexeme_char lexbuf (i+1)) - 48) +
                (Char.code(Lexing.lexeme_char lexbuf (i+2)) - 48) in
  if (c < 0 || c > 255) && not (in_comment ())
  then raise (Error(Illegal_escape (Lexing.lexeme lexbuf),
                    Location.curr lexbuf))
  else Char.chr c

let char_for_hexadecimal_code lexbuf i =
  let d1 = Char.code (Lexing.lexeme_char lexbuf i) in
  let val1 = if d1 >= 97 then d1 - 87
             else if d1 >= 65 then d1 - 55
             else d1 - 48
  in
  let d2 = Char.code (Lexing.lexeme_char lexbuf (i+1)) in
  let val2 = if d2 >= 97 then d2 - 87
             else if d2 >= 65 then d2 - 55
             else d2 - 48
  in
  Char.chr (val1 * 16 + val2)

(* Remove underscores from float literals *)

let remove_underscores s =
  let l = String.length s in
  let rec remove src dst =
    if src >= l then
      if dst >= l then s else String.sub s 0 dst
    else
      match s.[src] with
        '_' -> remove (src + 1) dst
      |  c  -> s.[dst] <- c; remove (src + 1) (dst + 1)
  in remove 0 0

(* Update the current location with file name and line number. *)

let update_loc lexbuf file line absolute chars =
  let pos = lexbuf.lex_curr_p in
  let new_file = match file with
                 | None -> pos.pos_fname
                 | Some s -> s
  in
  lexbuf.lex_curr_p <- { pos with
    pos_fname = new_file;
    pos_lnum = if absolute then line else pos.pos_lnum + line;
    pos_bol = pos.pos_cnum - chars;
  }
;;

(* Error report *)

open Format

let report_error ppf = function
  | Illegal_character c ->
      fprintf ppf "Illegal character (%s)" (Char.escaped c)
  | Illegal_escape s ->
      fprintf ppf "Illegal backslash escape in string or character (%s)" s
  | Unterminated_comment ->
      fprintf ppf "Comment not terminated"
  | Unterminated_string ->
      fprintf ppf "String literal not terminated"
  | Unterminated_string_in_comment ->
      fprintf ppf "This comment contains an unterminated string literal"
  | Keyword_as_label kwd ->
      fprintf ppf "`%s' is a keyword, it cannot be used as label name" kwd
  | Literal_overflow ty ->
      fprintf ppf "Integer literal exceeds the range of representable integers of type %s" ty
  | Unknown_backslash_identifer id ->
      fprintf ppf "Unknown annotation identifier (%s)" id
  | Not_in_annotation ->
      fprintf ppf "Not in an annotation"
;;


# 218 "ml/parsing/lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\176\255\177\255\224\000\003\001\038\001\073\001\108\001\
    \193\255\143\001\180\001\067\000\201\255\091\000\217\001\252\001\
    \069\000\071\000\084\000\031\002\218\255\220\255\223\255\066\002\
    \122\000\101\002\092\000\047\001\238\255\120\002\153\002\226\002\
    \178\003\130\004\097\005\189\005\141\006\126\000\127\000\015\001\
    \254\255\026\001\108\007\138\007\250\255\090\008\057\009\247\255\
    \239\255\240\255\241\255\112\000\045\003\092\000\094\000\055\003\
    \253\003\205\004\103\002\213\004\132\000\074\009\111\000\237\000\
    \113\000\237\255\236\255\232\255\049\006\078\003\117\000\235\255\
    \026\004\151\000\234\255\216\006\219\000\233\255\234\000\229\255\
    \231\255\099\009\228\255\252\007\122\005\001\000\227\255\007\000\
    \115\001\150\001\008\000\004\000\227\255\166\009\201\009\238\009\
    \017\010\215\255\211\255\212\255\213\255\209\255\052\010\087\010\
    \122\010\157\010\192\010\227\010\202\255\203\255\204\255\199\255\
    \196\255\006\011\192\255\194\255\041\011\076\011\103\003\244\255\
    \005\000\245\255\246\255\062\002\253\255\218\000\223\000\255\255\
    \254\255\094\011\229\000\012\001\233\000\252\255\251\255\222\007\
    \223\004\234\000\249\255\234\004\235\000\248\255\165\008\238\000\
    \029\005\006\000\167\011\201\011\132\005\024\001\045\001\189\005\
    \240\011\244\000\248\000\027\001\045\001\010\001\049\001\009\001\
    \009\000";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\077\000\074\000\073\000\068\000\071\000\
    \255\255\060\000\057\000\055\000\255\255\050\000\049\000\047\000\
    \045\000\041\000\039\000\069\000\255\255\255\255\255\255\030\000\
    \029\000\036\000\034\000\033\000\255\255\012\000\012\000\011\000\
    \010\000\009\000\006\000\004\000\003\000\002\000\000\000\000\000\
    \255\255\255\255\072\000\255\255\255\255\255\255\007\000\255\255\
    \255\255\255\255\255\255\255\255\013\000\255\255\255\255\255\255\
    \012\000\012\000\012\000\013\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\023\000\023\000\023\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\025\000\255\255\
    \255\255\076\000\255\255\255\255\255\255\028\000\255\255\255\255\
    \255\255\255\255\255\255\028\000\255\255\031\000\075\000\070\000\
    \038\000\255\255\255\255\255\255\255\255\255\255\073\000\048\000\
    \066\000\065\000\073\000\064\000\255\255\255\255\255\255\255\255\
    \255\255\058\000\255\255\255\255\071\000\067\000\255\255\255\255\
    \010\000\255\255\255\255\011\000\255\255\011\000\011\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\006\000\008\000\005\000\005\000\001\000\001\000\255\255\
    \255\255\002\000\255\255\255\255\001\000\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_default = 
   "\001\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\000\000\255\255\
    \255\255\255\255\255\255\062\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\255\255\000\000\255\255\255\255\000\000\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\067\000\255\255\255\255\
    \255\255\000\000\000\000\000\000\255\255\255\255\255\255\000\000\
    \255\255\255\255\000\000\255\255\255\255\000\000\255\255\000\000\
    \000\000\255\255\000\000\087\000\255\255\255\255\000\000\087\000\
    \088\000\087\000\090\000\255\255\000\000\255\255\255\255\255\255\
    \255\255\000\000\000\000\000\000\000\000\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\000\000\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\255\255\255\255\119\000\000\000\
    \255\255\000\000\000\000\130\000\000\000\255\255\255\255\000\000\
    \000\000\255\255\255\255\255\255\255\255\000\000\000\000\255\255\
    \255\255\255\255\000\000\255\255\255\255\000\000\255\255\255\255\
    \047\000\255\255\044\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\155\000\158\000\255\255\158\000\255\255\
    \255\255";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\037\000\039\000\086\000\037\000\038\000\092\000\121\000\
    \138\000\086\000\092\000\127\000\085\000\091\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \037\000\007\000\028\000\024\000\005\000\003\000\023\000\027\000\
    \026\000\021\000\025\000\006\000\020\000\019\000\018\000\003\000\
    \030\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
    \029\000\029\000\017\000\016\000\015\000\014\000\009\000\034\000\
    \004\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\013\000\032\000\012\000\004\000\036\000\
    \022\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\011\000\010\000\008\000\035\000\111\000\
    \101\000\100\000\097\000\084\000\099\000\098\000\078\000\037\000\
    \041\000\039\000\037\000\041\000\058\000\058\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\057\000\066\000\109\000\
    \065\000\108\000\084\000\060\000\071\000\060\000\037\000\041\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\074\000\040\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\110\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\064\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \002\000\003\000\077\000\128\000\003\000\003\000\003\000\255\255\
    \255\255\127\000\003\000\003\000\044\000\003\000\003\000\003\000\
    \134\000\138\000\141\000\079\000\065\000\047\000\132\000\154\000\
    \041\000\155\000\003\000\041\000\003\000\003\000\003\000\003\000\
    \003\000\150\000\150\000\041\000\004\000\156\000\041\000\004\000\
    \004\000\004\000\080\000\159\000\160\000\004\000\004\000\041\000\
    \004\000\004\000\004\000\134\000\000\000\000\000\150\000\157\000\
    \150\000\064\000\041\000\157\000\063\000\004\000\003\000\004\000\
    \004\000\004\000\004\000\004\000\000\000\000\000\000\000\005\000\
    \000\000\000\000\005\000\005\000\005\000\150\000\000\000\040\000\
    \005\000\005\000\000\000\005\000\005\000\005\000\255\255\000\000\
    \000\000\000\000\040\000\000\000\003\000\000\000\003\000\000\000\
    \005\000\004\000\005\000\005\000\005\000\005\000\005\000\000\000\
    \000\000\000\000\094\000\000\000\000\000\094\000\094\000\094\000\
    \000\000\000\000\000\000\094\000\094\000\000\000\094\000\094\000\
    \094\000\000\000\000\000\000\000\000\000\086\000\000\000\004\000\
    \085\000\004\000\000\000\094\000\005\000\094\000\094\000\094\000\
    \094\000\094\000\000\000\061\000\000\000\116\000\000\000\000\000\
    \116\000\116\000\116\000\000\000\000\000\090\000\116\000\116\000\
    \000\000\116\000\116\000\116\000\000\000\000\000\000\000\089\000\
    \086\000\000\000\005\000\085\000\005\000\000\000\116\000\094\000\
    \116\000\117\000\116\000\116\000\116\000\000\000\000\000\000\000\
    \005\000\000\000\000\000\005\000\005\000\005\000\089\000\000\000\
    \088\000\005\000\005\000\000\000\005\000\005\000\005\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\094\000\000\000\094\000\
    \000\000\005\000\116\000\005\000\005\000\005\000\005\000\005\000\
    \000\000\000\000\000\000\000\000\000\000\005\000\000\000\000\000\
    \005\000\005\000\005\000\000\000\000\000\000\000\005\000\005\000\
    \000\000\005\000\005\000\005\000\000\000\000\000\000\000\000\000\
    \116\000\000\000\116\000\000\000\115\000\005\000\005\000\000\000\
    \005\000\005\000\005\000\005\000\005\000\000\000\000\000\000\000\
    \000\000\000\000\005\000\000\000\000\000\005\000\005\000\005\000\
    \000\000\000\000\000\000\005\000\005\000\000\000\005\000\005\000\
    \005\000\000\000\000\000\005\000\114\000\005\000\000\000\000\000\
    \000\000\112\000\005\000\005\000\000\000\005\000\106\000\005\000\
    \005\000\005\000\000\000\255\255\000\000\005\000\000\000\000\000\
    \005\000\005\000\005\000\000\000\000\000\000\000\005\000\005\000\
    \000\000\103\000\005\000\005\000\000\000\255\255\000\000\255\255\
    \113\000\255\255\005\000\000\000\000\000\000\000\005\000\005\000\
    \005\000\102\000\005\000\005\000\005\000\000\000\000\000\000\000\
    \094\000\000\000\000\000\094\000\094\000\094\000\000\000\000\000\
    \132\000\094\000\094\000\131\000\094\000\095\000\094\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\005\000\000\000\005\000\
    \000\000\094\000\005\000\094\000\094\000\096\000\094\000\094\000\
    \000\000\000\000\000\000\005\000\000\000\133\000\005\000\005\000\
    \093\000\000\000\000\000\000\000\005\000\005\000\000\000\005\000\
    \005\000\005\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \005\000\000\000\005\000\000\000\005\000\094\000\005\000\005\000\
    \005\000\005\000\005\000\000\000\000\000\000\000\003\000\000\000\
    \000\000\003\000\003\000\003\000\000\000\000\000\082\000\081\000\
    \003\000\000\000\003\000\003\000\003\000\000\000\255\255\058\000\
    \058\000\000\000\129\000\094\000\000\000\094\000\000\000\003\000\
    \005\000\003\000\003\000\003\000\003\000\003\000\052\000\000\000\
    \029\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
    \029\000\029\000\000\000\049\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\051\000\005\000\000\000\
    \005\000\000\000\000\000\003\000\049\000\000\000\058\000\052\000\
    \000\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
    \029\000\029\000\029\000\050\000\000\000\048\000\000\000\029\000\
    \000\000\000\000\000\000\053\000\000\000\051\000\051\000\000\000\
    \000\000\003\000\000\000\003\000\050\000\049\000\048\000\000\000\
    \054\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\055\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \029\000\000\000\000\000\053\000\000\000\000\000\051\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\050\000\000\000\048\000\
    \054\000\031\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\055\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\000\000\255\255\000\000\
    \000\000\031\000\000\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\052\000\052\000\052\000\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\121\000\051\000\000\000\120\000\000\000\000\000\000\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\072\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\072\000\072\000\
    \000\000\124\000\000\000\000\000\052\000\000\000\123\000\126\000\
    \000\000\125\000\051\000\000\000\000\000\000\000\000\000\000\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\000\000\000\000\
    \000\000\000\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\000\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\032\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\000\000\000\000\000\000\
    \000\000\032\000\000\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\000\000\000\000\000\000\000\000\
    \000\000\049\000\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\073\000\073\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\056\000\000\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\000\000\000\000\000\000\122\000\
    \000\000\050\000\000\000\048\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\000\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\033\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\000\000\000\000\000\000\
    \000\000\033\000\000\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\057\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\139\000\
    \139\000\139\000\139\000\139\000\139\000\139\000\139\000\139\000\
    \139\000\049\000\140\000\140\000\140\000\140\000\140\000\140\000\
    \140\000\140\000\140\000\140\000\000\000\000\000\000\000\138\000\
    \000\000\000\000\145\000\000\000\057\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\059\000\000\000\000\000\000\000\
    \000\000\050\000\000\000\048\000\000\000\000\000\000\000\127\000\
    \000\000\000\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\000\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\146\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\042\000\084\000\000\000\042\000\042\000\042\000\
    \000\000\000\000\000\000\042\000\042\000\000\000\042\000\042\000\
    \042\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\084\000\042\000\000\000\042\000\042\000\042\000\
    \046\000\042\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\151\000\151\000\151\000\151\000\
    \151\000\151\000\151\000\151\000\151\000\151\000\000\000\042\000\
    \045\000\000\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\000\000\042\000\042\000\042\000\
    \000\000\042\000\042\000\042\000\000\000\000\000\000\000\042\000\
    \042\000\000\000\042\000\042\000\042\000\133\000\133\000\133\000\
    \133\000\133\000\133\000\133\000\133\000\133\000\133\000\042\000\
    \000\000\042\000\042\000\042\000\042\000\042\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\042\000\043\000\141\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \000\000\042\000\000\000\042\000\000\000\000\000\000\000\000\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \000\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\075\000\075\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \000\000\000\000\000\000\000\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\033\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \000\000\000\000\000\000\000\000\033\000\000\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \076\000\076\000\076\000\076\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\076\000\076\000\076\000\076\000\076\000\076\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\076\000\076\000\076\000\076\000\076\000\076\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\000\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\000\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\042\000\000\000\000\000\
    \042\000\042\000\042\000\000\000\000\000\000\000\042\000\042\000\
    \000\000\042\000\042\000\042\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\042\000\000\000\
    \042\000\042\000\042\000\042\000\042\000\000\000\000\000\000\000\
    \000\000\043\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\044\000\000\000\000\000\000\000\
    \000\000\000\000\042\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\000\000\000\000\000\000\
    \042\000\043\000\042\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\089\000\086\000\000\000\
    \000\000\085\000\000\000\000\000\000\000\000\000\142\000\142\000\
    \142\000\142\000\142\000\142\000\142\000\142\000\142\000\142\000\
    \000\000\000\000\000\000\000\000\089\000\000\000\088\000\142\000\
    \142\000\142\000\142\000\142\000\142\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\142\000\
    \142\000\142\000\142\000\142\000\142\000\000\000\000\000\000\000\
    \000\000\000\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\000\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\045\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\047\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\000\000\000\000\000\000\
    \000\000\045\000\000\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\143\000\143\000\143\000\
    \143\000\143\000\143\000\143\000\143\000\143\000\143\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\143\000\143\000\
    \143\000\143\000\143\000\143\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\143\000\143\000\
    \143\000\143\000\143\000\143\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\000\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\000\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\042\000\000\000\000\000\042\000\042\000\042\000\
    \000\000\000\000\000\000\042\000\042\000\000\000\042\000\042\000\
    \042\000\000\000\070\000\000\000\070\000\000\000\000\000\000\000\
    \000\000\070\000\000\000\042\000\000\000\042\000\042\000\042\000\
    \042\000\042\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\069\000\069\000\081\000\000\000\000\000\081\000\
    \081\000\081\000\000\000\000\000\000\000\081\000\081\000\000\000\
    \081\000\081\000\081\000\000\000\000\000\000\000\000\000\042\000\
    \000\000\000\000\000\000\000\000\000\000\081\000\000\000\081\000\
    \081\000\081\000\081\000\081\000\000\000\000\000\070\000\000\000\
    \000\000\000\000\000\000\000\000\070\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\042\000\000\000\042\000\
    \070\000\000\000\000\000\000\000\070\000\000\000\070\000\000\000\
    \000\000\081\000\068\000\000\000\000\000\000\000\000\000\005\000\
    \000\000\000\000\005\000\005\000\005\000\000\000\000\000\000\000\
    \005\000\005\000\000\000\005\000\005\000\005\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\081\000\
    \005\000\081\000\005\000\005\000\005\000\005\000\005\000\000\000\
    \000\000\000\000\094\000\000\000\000\000\094\000\094\000\094\000\
    \000\000\000\000\000\000\094\000\094\000\000\000\094\000\094\000\
    \094\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\094\000\005\000\094\000\094\000\094\000\
    \094\000\094\000\000\000\000\000\000\000\000\000\000\000\094\000\
    \000\000\000\000\094\000\094\000\094\000\000\000\000\000\000\000\
    \094\000\094\000\000\000\094\000\094\000\094\000\000\000\000\000\
    \000\000\000\000\005\000\000\000\005\000\000\000\000\000\094\000\
    \094\000\000\000\094\000\094\000\094\000\094\000\094\000\000\000\
    \000\000\000\000\094\000\000\000\000\000\094\000\094\000\094\000\
    \000\000\000\000\000\000\094\000\094\000\000\000\094\000\094\000\
    \094\000\000\000\000\000\000\000\000\000\094\000\000\000\094\000\
    \000\000\000\000\255\255\094\000\094\000\094\000\094\000\094\000\
    \094\000\094\000\000\000\000\000\000\000\005\000\000\000\000\000\
    \005\000\005\000\005\000\000\000\000\000\000\000\005\000\005\000\
    \000\000\005\000\005\000\005\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\094\000\000\000\094\000\000\000\005\000\094\000\
    \005\000\105\000\104\000\005\000\005\000\000\000\000\000\000\000\
    \005\000\000\000\000\000\005\000\005\000\005\000\000\000\000\000\
    \000\000\005\000\005\000\000\000\005\000\005\000\005\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\094\000\000\000\094\000\
    \000\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \000\000\000\000\000\000\005\000\000\000\000\000\005\000\005\000\
    \005\000\000\000\000\000\000\000\005\000\005\000\000\000\005\000\
    \005\000\005\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \005\000\000\000\005\000\000\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\000\000\000\000\000\000\005\000\000\000\
    \000\000\005\000\005\000\005\000\000\000\000\000\000\000\005\000\
    \005\000\000\000\005\000\005\000\005\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\005\000\000\000\005\000\000\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\000\000\000\000\
    \000\000\005\000\000\000\000\000\005\000\005\000\005\000\000\000\
    \000\000\000\000\005\000\005\000\000\000\005\000\005\000\005\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\005\000\000\000\
    \005\000\000\000\005\000\005\000\005\000\005\000\107\000\005\000\
    \005\000\000\000\000\000\000\000\005\000\000\000\000\000\005\000\
    \005\000\005\000\000\000\000\000\000\000\005\000\005\000\000\000\
    \005\000\005\000\005\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\005\000\000\000\005\000\000\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\000\000\000\000\000\000\005\000\
    \000\000\000\000\005\000\005\000\005\000\000\000\000\000\000\000\
    \005\000\005\000\000\000\005\000\005\000\005\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\005\000\000\000\005\000\000\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\000\000\
    \000\000\000\000\116\000\000\000\000\000\116\000\116\000\116\000\
    \000\000\000\000\000\000\116\000\116\000\000\000\116\000\116\000\
    \116\000\000\000\000\000\000\000\000\000\000\000\000\000\005\000\
    \000\000\005\000\000\000\116\000\005\000\116\000\116\000\116\000\
    \116\000\116\000\000\000\000\000\000\000\116\000\000\000\000\000\
    \116\000\116\000\116\000\000\000\000\000\000\000\116\000\116\000\
    \000\000\116\000\116\000\116\000\000\000\000\000\137\000\000\000\
    \137\000\000\000\005\000\000\000\005\000\137\000\116\000\116\000\
    \116\000\116\000\116\000\116\000\116\000\000\000\136\000\136\000\
    \136\000\136\000\136\000\136\000\136\000\136\000\136\000\136\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\116\000\000\000\116\000\
    \000\000\000\000\116\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\150\000\000\000\000\000\149\000\000\000\000\000\000\000\
    \000\000\000\000\137\000\000\000\000\000\000\000\000\000\000\000\
    \137\000\000\000\000\000\000\000\000\000\000\000\000\000\124\000\
    \116\000\124\000\116\000\000\000\137\000\000\000\124\000\000\000\
    \137\000\000\000\137\000\000\000\000\000\000\000\135\000\148\000\
    \148\000\148\000\148\000\148\000\148\000\148\000\148\000\148\000\
    \148\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\152\000\152\000\152\000\152\000\152\000\152\000\152\000\
    \152\000\152\000\152\000\124\000\000\000\000\000\000\000\000\000\
    \000\000\124\000\152\000\152\000\152\000\152\000\152\000\152\000\
    \000\000\000\000\000\000\000\000\000\000\124\000\000\000\000\000\
    \000\000\124\000\000\000\124\000\000\000\000\000\000\000\147\000\
    \134\000\134\000\134\000\134\000\134\000\134\000\134\000\134\000\
    \134\000\134\000\152\000\152\000\152\000\152\000\152\000\152\000\
    \000\000\134\000\134\000\134\000\134\000\134\000\134\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\134\000\134\000\134\000\134\000\134\000\134\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\085\000\000\000\000\000\091\000\120\000\
    \145\000\087\000\090\000\160\000\087\000\090\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\011\000\
    \016\000\017\000\018\000\024\000\017\000\017\000\026\000\037\000\
    \038\000\038\000\037\000\038\000\053\000\053\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\062\000\013\000\
    \064\000\013\000\024\000\051\000\070\000\051\000\037\000\038\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\051\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\060\000\060\000\060\000\060\000\
    \060\000\060\000\060\000\060\000\060\000\060\000\073\000\038\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\013\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\063\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\003\000\076\000\125\000\003\000\003\000\003\000\087\000\
    \090\000\126\000\003\000\003\000\130\000\003\000\003\000\003\000\
    \132\000\137\000\140\000\078\000\063\000\143\000\131\000\153\000\
    \039\000\154\000\003\000\039\000\003\000\003\000\003\000\003\000\
    \003\000\149\000\149\000\041\000\004\000\155\000\041\000\004\000\
    \004\000\004\000\078\000\157\000\159\000\004\000\004\000\039\000\
    \004\000\004\000\004\000\131\000\255\255\255\255\150\000\156\000\
    \149\000\027\000\041\000\158\000\027\000\004\000\003\000\004\000\
    \004\000\004\000\004\000\004\000\255\255\255\255\255\255\005\000\
    \255\255\255\255\005\000\005\000\005\000\150\000\255\255\039\000\
    \005\000\005\000\255\255\005\000\005\000\005\000\027\000\255\255\
    \255\255\255\255\041\000\255\255\003\000\255\255\003\000\255\255\
    \005\000\004\000\005\000\005\000\005\000\005\000\005\000\255\255\
    \255\255\255\255\006\000\255\255\255\255\006\000\006\000\006\000\
    \255\255\255\255\255\255\006\000\006\000\255\255\006\000\006\000\
    \006\000\255\255\255\255\255\255\255\255\088\000\255\255\004\000\
    \088\000\004\000\255\255\006\000\005\000\006\000\006\000\006\000\
    \006\000\006\000\255\255\027\000\255\255\007\000\255\255\255\255\
    \007\000\007\000\007\000\255\255\255\255\088\000\007\000\007\000\
    \255\255\007\000\007\000\007\000\255\255\255\255\255\255\089\000\
    \089\000\255\255\005\000\089\000\005\000\255\255\007\000\006\000\
    \007\000\007\000\007\000\007\000\007\000\255\255\255\255\255\255\
    \009\000\255\255\255\255\009\000\009\000\009\000\089\000\255\255\
    \089\000\009\000\009\000\255\255\009\000\009\000\009\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\006\000\255\255\006\000\
    \255\255\009\000\007\000\009\000\009\000\009\000\009\000\009\000\
    \255\255\255\255\255\255\255\255\255\255\010\000\255\255\255\255\
    \010\000\010\000\010\000\255\255\255\255\255\255\010\000\010\000\
    \255\255\010\000\010\000\010\000\255\255\255\255\255\255\255\255\
    \007\000\255\255\007\000\255\255\009\000\009\000\010\000\255\255\
    \010\000\010\000\010\000\010\000\010\000\255\255\255\255\255\255\
    \255\255\255\255\014\000\255\255\255\255\014\000\014\000\014\000\
    \255\255\255\255\255\255\014\000\014\000\255\255\014\000\014\000\
    \014\000\255\255\255\255\009\000\009\000\009\000\255\255\255\255\
    \255\255\010\000\010\000\014\000\255\255\014\000\014\000\014\000\
    \014\000\014\000\255\255\155\000\255\255\015\000\255\255\255\255\
    \015\000\015\000\015\000\255\255\255\255\255\255\015\000\015\000\
    \255\255\015\000\015\000\015\000\255\255\156\000\255\255\027\000\
    \010\000\158\000\010\000\255\255\255\255\255\255\015\000\014\000\
    \015\000\015\000\015\000\015\000\015\000\255\255\255\255\255\255\
    \019\000\255\255\255\255\019\000\019\000\019\000\255\255\255\255\
    \123\000\019\000\019\000\123\000\019\000\019\000\019\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\014\000\255\255\014\000\
    \255\255\019\000\015\000\019\000\019\000\019\000\019\000\019\000\
    \255\255\255\255\255\255\023\000\255\255\123\000\023\000\023\000\
    \023\000\255\255\255\255\255\255\023\000\023\000\255\255\023\000\
    \023\000\023\000\255\255\088\000\255\255\255\255\255\255\255\255\
    \015\000\255\255\015\000\255\255\023\000\019\000\023\000\023\000\
    \023\000\023\000\023\000\255\255\255\255\255\255\025\000\255\255\
    \255\255\025\000\025\000\025\000\255\255\255\255\025\000\025\000\
    \025\000\255\255\025\000\025\000\025\000\255\255\089\000\058\000\
    \058\000\255\255\123\000\019\000\255\255\019\000\255\255\025\000\
    \023\000\025\000\025\000\025\000\025\000\025\000\029\000\255\255\
    \029\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
    \029\000\029\000\255\255\058\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\029\000\023\000\255\255\
    \023\000\255\255\255\255\025\000\029\000\255\255\058\000\030\000\
    \255\255\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\058\000\255\255\058\000\255\255\029\000\
    \255\255\255\255\255\255\030\000\255\255\029\000\030\000\255\255\
    \255\255\025\000\255\255\025\000\029\000\030\000\029\000\255\255\
    \030\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\030\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \030\000\255\255\255\255\030\000\255\255\255\255\030\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\030\000\255\255\030\000\
    \030\000\031\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\030\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\255\255\123\000\255\255\
    \255\255\031\000\255\255\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\052\000\052\000\052\000\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\118\000\052\000\255\255\118\000\255\255\255\255\255\255\
    \055\000\055\000\055\000\055\000\055\000\055\000\069\000\069\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \255\255\118\000\255\255\255\255\052\000\255\255\118\000\118\000\
    \255\255\118\000\052\000\255\255\255\255\255\255\255\255\255\255\
    \055\000\055\000\055\000\055\000\055\000\055\000\255\255\255\255\
    \255\255\255\255\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\255\255\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\032\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\255\255\255\255\255\255\
    \255\255\032\000\255\255\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\056\000\056\000\
    \056\000\056\000\056\000\056\000\255\255\255\255\255\255\255\255\
    \255\255\056\000\072\000\072\000\072\000\072\000\072\000\072\000\
    \072\000\072\000\072\000\072\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\056\000\255\255\056\000\056\000\
    \056\000\056\000\056\000\056\000\255\255\255\255\255\255\118\000\
    \255\255\056\000\255\255\056\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\255\255\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\033\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\255\255\255\255\255\255\
    \255\255\033\000\255\255\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\057\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\136\000\
    \136\000\136\000\136\000\136\000\136\000\136\000\136\000\136\000\
    \136\000\057\000\139\000\139\000\139\000\139\000\139\000\139\000\
    \139\000\139\000\139\000\139\000\255\255\255\255\255\255\144\000\
    \255\255\255\255\144\000\255\255\057\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\059\000\255\255\255\255\255\255\
    \255\255\057\000\255\255\057\000\255\255\255\255\255\255\144\000\
    \255\255\255\255\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\255\255\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\144\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\034\000\084\000\255\255\034\000\034\000\034\000\
    \255\255\255\255\255\255\034\000\034\000\255\255\034\000\034\000\
    \034\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\084\000\034\000\255\255\034\000\034\000\034\000\
    \034\000\034\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\148\000\148\000\148\000\148\000\
    \148\000\148\000\148\000\148\000\148\000\148\000\255\255\034\000\
    \034\000\255\255\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\255\255\034\000\035\000\034\000\
    \255\255\035\000\035\000\035\000\255\255\255\255\255\255\035\000\
    \035\000\255\255\035\000\035\000\035\000\151\000\151\000\151\000\
    \151\000\151\000\151\000\151\000\151\000\151\000\151\000\035\000\
    \255\255\035\000\035\000\035\000\035\000\035\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\035\000\035\000\144\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \255\255\035\000\255\255\035\000\255\255\255\255\255\255\255\255\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \255\255\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\068\000\068\000\068\000\068\000\068\000\068\000\068\000\
    \068\000\068\000\068\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\068\000\068\000\068\000\068\000\068\000\068\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\068\000\068\000\068\000\068\000\068\000\068\000\
    \255\255\255\255\255\255\255\255\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\036\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \255\255\255\255\255\255\255\255\036\000\255\255\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\075\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\075\000\075\000\075\000\075\000\075\000\075\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\075\000\075\000\075\000\075\000\075\000\075\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\255\255\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\255\255\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\042\000\255\255\255\255\
    \042\000\042\000\042\000\255\255\255\255\255\255\042\000\042\000\
    \255\255\042\000\042\000\042\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\042\000\255\255\
    \042\000\042\000\042\000\042\000\042\000\255\255\255\255\255\255\
    \255\255\043\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\255\255\255\255\255\255\
    \255\255\255\255\042\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\255\255\255\255\255\255\
    \042\000\043\000\042\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\083\000\083\000\255\255\
    \255\255\083\000\255\255\255\255\255\255\255\255\135\000\135\000\
    \135\000\135\000\135\000\135\000\135\000\135\000\135\000\135\000\
    \255\255\255\255\255\255\255\255\083\000\255\255\083\000\135\000\
    \135\000\135\000\135\000\135\000\135\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\135\000\
    \135\000\135\000\135\000\135\000\135\000\255\255\255\255\255\255\
    \255\255\255\255\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\255\255\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\045\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\255\255\255\255\255\255\
    \255\255\045\000\255\255\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\142\000\142\000\142\000\
    \142\000\142\000\142\000\142\000\142\000\142\000\142\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\142\000\142\000\
    \142\000\142\000\142\000\142\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\083\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\142\000\142\000\
    \142\000\142\000\142\000\142\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\255\255\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\255\255\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\046\000\255\255\255\255\046\000\046\000\046\000\
    \255\255\255\255\255\255\046\000\046\000\255\255\046\000\046\000\
    \046\000\255\255\061\000\255\255\061\000\255\255\255\255\255\255\
    \255\255\061\000\255\255\046\000\255\255\046\000\046\000\046\000\
    \046\000\046\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\081\000\255\255\255\255\081\000\
    \081\000\081\000\255\255\255\255\255\255\081\000\081\000\255\255\
    \081\000\081\000\081\000\255\255\255\255\255\255\255\255\046\000\
    \255\255\255\255\255\255\255\255\255\255\081\000\255\255\081\000\
    \081\000\081\000\081\000\081\000\255\255\255\255\061\000\255\255\
    \255\255\255\255\255\255\255\255\061\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\046\000\255\255\046\000\
    \061\000\255\255\255\255\255\255\061\000\255\255\061\000\255\255\
    \255\255\081\000\061\000\255\255\255\255\255\255\255\255\093\000\
    \255\255\255\255\093\000\093\000\093\000\255\255\255\255\255\255\
    \093\000\093\000\255\255\093\000\093\000\093\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\081\000\
    \093\000\081\000\093\000\093\000\093\000\093\000\093\000\255\255\
    \255\255\255\255\094\000\255\255\255\255\094\000\094\000\094\000\
    \255\255\255\255\255\255\094\000\094\000\255\255\094\000\094\000\
    \094\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\094\000\093\000\094\000\094\000\094\000\
    \094\000\094\000\255\255\255\255\255\255\255\255\255\255\095\000\
    \255\255\255\255\095\000\095\000\095\000\255\255\255\255\255\255\
    \095\000\095\000\255\255\095\000\095\000\095\000\255\255\255\255\
    \255\255\255\255\093\000\255\255\093\000\255\255\255\255\094\000\
    \095\000\255\255\095\000\095\000\095\000\095\000\095\000\255\255\
    \255\255\255\255\096\000\255\255\255\255\096\000\096\000\096\000\
    \255\255\255\255\255\255\096\000\096\000\255\255\096\000\096\000\
    \096\000\255\255\255\255\255\255\255\255\094\000\255\255\094\000\
    \255\255\255\255\061\000\096\000\095\000\096\000\096\000\096\000\
    \096\000\096\000\255\255\255\255\255\255\102\000\255\255\255\255\
    \102\000\102\000\102\000\255\255\255\255\255\255\102\000\102\000\
    \255\255\102\000\102\000\102\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\095\000\255\255\095\000\255\255\102\000\096\000\
    \102\000\102\000\102\000\102\000\102\000\255\255\255\255\255\255\
    \103\000\255\255\255\255\103\000\103\000\103\000\255\255\255\255\
    \255\255\103\000\103\000\255\255\103\000\103\000\103\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\096\000\255\255\096\000\
    \255\255\103\000\102\000\103\000\103\000\103\000\103\000\103\000\
    \255\255\255\255\255\255\104\000\255\255\255\255\104\000\104\000\
    \104\000\255\255\255\255\255\255\104\000\104\000\255\255\104\000\
    \104\000\104\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \102\000\255\255\102\000\255\255\104\000\103\000\104\000\104\000\
    \104\000\104\000\104\000\255\255\255\255\255\255\105\000\255\255\
    \255\255\105\000\105\000\105\000\255\255\255\255\255\255\105\000\
    \105\000\255\255\105\000\105\000\105\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\103\000\255\255\103\000\255\255\105\000\
    \104\000\105\000\105\000\105\000\105\000\105\000\255\255\255\255\
    \255\255\106\000\255\255\255\255\106\000\106\000\106\000\255\255\
    \255\255\255\255\106\000\106\000\255\255\106\000\106\000\106\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\104\000\255\255\
    \104\000\255\255\106\000\105\000\106\000\106\000\106\000\106\000\
    \106\000\255\255\255\255\255\255\107\000\255\255\255\255\107\000\
    \107\000\107\000\255\255\255\255\255\255\107\000\107\000\255\255\
    \107\000\107\000\107\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\105\000\255\255\105\000\255\255\107\000\106\000\107\000\
    \107\000\107\000\107\000\107\000\255\255\255\255\255\255\113\000\
    \255\255\255\255\113\000\113\000\113\000\255\255\255\255\255\255\
    \113\000\113\000\255\255\113\000\113\000\113\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\106\000\255\255\106\000\255\255\
    \113\000\107\000\113\000\113\000\113\000\113\000\113\000\255\255\
    \255\255\255\255\116\000\255\255\255\255\116\000\116\000\116\000\
    \255\255\255\255\255\255\116\000\116\000\255\255\116\000\116\000\
    \116\000\255\255\255\255\255\255\255\255\255\255\255\255\107\000\
    \255\255\107\000\255\255\116\000\113\000\116\000\116\000\116\000\
    \116\000\116\000\255\255\255\255\255\255\117\000\255\255\255\255\
    \117\000\117\000\117\000\255\255\255\255\255\255\117\000\117\000\
    \255\255\117\000\117\000\117\000\255\255\255\255\129\000\255\255\
    \129\000\255\255\113\000\255\255\113\000\129\000\117\000\116\000\
    \117\000\117\000\117\000\117\000\117\000\255\255\129\000\129\000\
    \129\000\129\000\129\000\129\000\129\000\129\000\129\000\129\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\116\000\255\255\116\000\
    \255\255\255\255\117\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\146\000\255\255\255\255\146\000\255\255\255\255\255\255\
    \255\255\255\255\129\000\255\255\255\255\255\255\255\255\255\255\
    \129\000\255\255\255\255\255\255\255\255\255\255\255\255\146\000\
    \117\000\146\000\117\000\255\255\129\000\255\255\146\000\255\255\
    \129\000\255\255\129\000\255\255\255\255\255\255\129\000\146\000\
    \146\000\146\000\146\000\146\000\146\000\146\000\146\000\146\000\
    \146\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\147\000\147\000\147\000\147\000\147\000\147\000\147\000\
    \147\000\147\000\147\000\146\000\255\255\255\255\255\255\255\255\
    \255\255\146\000\147\000\147\000\147\000\147\000\147\000\147\000\
    \255\255\255\255\255\255\255\255\255\255\146\000\255\255\255\255\
    \255\255\146\000\255\255\146\000\255\255\255\255\255\255\146\000\
    \152\000\152\000\152\000\152\000\152\000\152\000\152\000\152\000\
    \152\000\152\000\147\000\147\000\147\000\147\000\147\000\147\000\
    \255\255\152\000\152\000\152\000\152\000\152\000\152\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\152\000\152\000\152\000\152\000\152\000\152\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\146\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_base_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\001\000\000\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\010\000\036\000\000\000\018\000\000\000\
    \002\000\000\000\000\000\000\000\033\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\004\000\000\000\000\000\006\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_backtrk_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\018\000\000\000\000\000\
    \000\000\000\000\000\000\033\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\045\000\045\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_default_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \025\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_trans_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\004\000\001\000\001\000\000\000\001\000\042\000\000\000\
    \042\000\042\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \004\000\000\000\028\000\004\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\013\000\004\000\000\000\000\000\
    \007\000\007\000\007\000\007\000\007\000\007\000\007\000\007\000\
    \007\000\007\000\007\000\007\000\007\000\007\000\007\000\007\000\
    \007\000\007\000\007\000\007\000\004\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\007\000\007\000\007\000\007\000\
    \007\000\007\000\007\000\007\000\007\000\007\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_check_code = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\024\000\000\000\038\000\088\000\000\000\146\000\088\000\
    \149\000\146\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \024\000\255\255\089\000\000\000\088\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\083\000\084\000\255\255\255\255\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\084\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\088\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_code = 
   "\255\005\255\255\004\255\255\006\255\255\000\005\255\008\255\007\
    \255\255\003\255\000\004\001\006\255\008\255\255\007\255\008\255\
    \255\000\004\001\006\003\007\002\008\255\001\255\255\000\001\255\
    ";
}

let rec token lexbuf =
  lexbuf.Lexing.lex_mem <- Array.create 9 (-1) ;   __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 258 "ml/parsing/lexer.mll"
      ( update_loc lexbuf None 1 false 0;
        token lexbuf
      )
# 1276 "ml/parsing/lexer.ml"

  | 1 ->
let
# 261 "ml/parsing/lexer.mll"
                        spaces
# 1282 "ml/parsing/lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) (lexbuf.Lexing.lex_curr_pos + -1) in
# 262 "ml/parsing/lexer.mll"
      ( if not !in_annotation then
	  raise (Error(Not_in_annotation, Location.curr lexbuf));
	update_loc lexbuf None 1 false (String.length spaces + 1);
	token lexbuf )
# 1289 "ml/parsing/lexer.ml"

  | 2 ->
# 267 "ml/parsing/lexer.mll"
      ( token lexbuf )
# 1294 "ml/parsing/lexer.ml"

  | 3 ->
# 269 "ml/parsing/lexer.mll"
      ( UNDERSCORE )
# 1299 "ml/parsing/lexer.ml"

  | 4 ->
# 270 "ml/parsing/lexer.mll"
         ( TILDE )
# 1304 "ml/parsing/lexer.ml"

  | 5 ->
# 272 "ml/parsing/lexer.mll"
      ( let s = Lexing.lexeme lexbuf in
        let name = String.sub s 1 (String.length s - 2) in
        if Hashtbl.mem keyword_table name then
          raise (Error(Keyword_as_label name, Location.curr lexbuf));
        LABEL name )
# 1313 "ml/parsing/lexer.ml"

  | 6 ->
# 277 "ml/parsing/lexer.mll"
         ( QUESTION )
# 1318 "ml/parsing/lexer.ml"

  | 7 ->
# 278 "ml/parsing/lexer.mll"
         ( QUESTIONQUESTION )
# 1323 "ml/parsing/lexer.ml"

  | 8 ->
# 280 "ml/parsing/lexer.mll"
      ( let s = Lexing.lexeme lexbuf in
        let name = String.sub s 1 (String.length s - 2) in
        if Hashtbl.mem keyword_table name then
          raise (Error(Keyword_as_label name, Location.curr lexbuf));
        OPTLABEL name )
# 1332 "ml/parsing/lexer.ml"

  | 9 ->
# 286 "ml/parsing/lexer.mll"
      ( let s = Lexing.lexeme lexbuf in
          try
            Hashtbl.find keyword_table s
          with Not_found ->
            if !in_annotation then
	      try Hashtbl.find annot_keyword_table s with Not_found -> LIDENT s
	    else LIDENT s )
# 1343 "ml/parsing/lexer.ml"

  | 10 ->
# 294 "ml/parsing/lexer.mll"
      ( if !in_annotation then
	  let s = Lexing.lexeme lexbuf in
	  try Hashtbl.find annot_keyword_table s with Not_found ->
	    raise (Error(Unknown_backslash_identifer s, Location.curr lexbuf))
	else
	  raise (Error(Not_in_annotation, Location.curr lexbuf)) )
# 1353 "ml/parsing/lexer.ml"

  | 11 ->
# 301 "ml/parsing/lexer.mll"
      ( UIDENT(Lexing.lexeme lexbuf) )
# 1358 "ml/parsing/lexer.ml"

  | 12 ->
# 303 "ml/parsing/lexer.mll"
      ( try
          INT (int_of_string(Lexing.lexeme lexbuf))
        with Failure _ ->
          raise (Error(Literal_overflow "int", Location.curr lexbuf))
      )
# 1367 "ml/parsing/lexer.ml"

  | 13 ->
# 309 "ml/parsing/lexer.mll"
      ( FLOAT (remove_underscores(Lexing.lexeme lexbuf)) )
# 1372 "ml/parsing/lexer.ml"

  | 14 ->
# 311 "ml/parsing/lexer.mll"
      ( let s = Lexing.lexeme lexbuf in
        try
          INT32 (Int32.of_string(String.sub s 0 (String.length s - 1)))
        with Failure _ ->
          raise (Error(Literal_overflow "int32", Location.curr lexbuf)) )
# 1381 "ml/parsing/lexer.ml"

  | 15 ->
# 317 "ml/parsing/lexer.mll"
      ( let s = Lexing.lexeme lexbuf in
        try
          INT64 (Int64.of_string(String.sub s 0 (String.length s - 1)))
        with Failure _ ->
          raise (Error(Literal_overflow "int64", Location.curr lexbuf)) )
# 1390 "ml/parsing/lexer.ml"

  | 16 ->
# 323 "ml/parsing/lexer.mll"
      ( let s = Lexing.lexeme lexbuf in
        try
          NATIVEINT
            (Nativeint.of_string(String.sub s 0 (String.length s - 1)))
        with Failure _ ->
          raise (Error(Literal_overflow "nativeint", Location.curr lexbuf)) )
# 1400 "ml/parsing/lexer.ml"

  | 17 ->
# 330 "ml/parsing/lexer.mll"
      ( reset_string_buffer();
        let string_start = lexbuf.lex_start_p in
        string_start_loc := Location.curr lexbuf;
        string lexbuf;
        lexbuf.lex_start_p <- string_start;
        STRING (get_stored_string()) )
# 1410 "ml/parsing/lexer.ml"

  | 18 ->
# 337 "ml/parsing/lexer.mll"
      ( update_loc lexbuf None 1 false 1;
        CHAR (Lexing.lexeme_char lexbuf 1) )
# 1416 "ml/parsing/lexer.ml"

  | 19 ->
# 340 "ml/parsing/lexer.mll"
      ( CHAR(Lexing.lexeme_char lexbuf 1) )
# 1421 "ml/parsing/lexer.ml"

  | 20 ->
# 342 "ml/parsing/lexer.mll"
      ( CHAR(char_for_backslash (Lexing.lexeme_char lexbuf 2)) )
# 1426 "ml/parsing/lexer.ml"

  | 21 ->
# 344 "ml/parsing/lexer.mll"
      ( CHAR(char_for_decimal_code lexbuf 2) )
# 1431 "ml/parsing/lexer.ml"

  | 22 ->
# 346 "ml/parsing/lexer.mll"
      ( CHAR(char_for_hexadecimal_code lexbuf 3) )
# 1436 "ml/parsing/lexer.ml"

  | 23 ->
# 348 "ml/parsing/lexer.mll"
      ( let l = Lexing.lexeme lexbuf in
        let esc = String.sub l 1 (String.length l - 1) in
        raise (Error(Illegal_escape esc, Location.curr lexbuf))
      )
# 1444 "ml/parsing/lexer.ml"

  | 24 ->
# 353 "ml/parsing/lexer.mll"
      ( in_annotation := true; LANNOT )
# 1449 "ml/parsing/lexer.ml"

  | 25 ->
# 355 "ml/parsing/lexer.mll"
      ( comment_start_loc := [Location.curr lexbuf];
        comment lexbuf;
        token lexbuf )
# 1456 "ml/parsing/lexer.ml"

  | 26 ->
# 359 "ml/parsing/lexer.mll"
      ( let loc = Location.curr lexbuf in
        Location.prerr_warning loc Warnings.Comment_start;
        comment_start_loc := [Location.curr lexbuf];
        comment lexbuf;
        token lexbuf
      )
# 1466 "ml/parsing/lexer.ml"

  | 27 ->
# 366 "ml/parsing/lexer.mll"
      ( if !in_annotation then begin
	  in_annotation := false;
	  RANNOT
	end else begin
	  let loc = Location.curr lexbuf in
          Location.prerr_warning loc Warnings.Comment_not_end;
          lexbuf.Lexing.lex_curr_pos <- lexbuf.Lexing.lex_curr_pos - 1;
          let curpos = lexbuf.lex_curr_p in
          lexbuf.lex_curr_p <- { curpos with pos_cnum = curpos.pos_cnum - 1 };
          STAR
	end
      )
# 1482 "ml/parsing/lexer.ml"

  | 28 ->
let
# 378 "ml/parsing/lexer.mll"
                                   num
# 1488 "ml/parsing/lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_mem.(1)
and
# 379 "ml/parsing/lexer.mll"
                                           name
# 1493 "ml/parsing/lexer.ml"
= Lexing.sub_lexeme_opt lexbuf lexbuf.Lexing.lex_mem.(3) lexbuf.Lexing.lex_mem.(2) in
# 381 "ml/parsing/lexer.mll"
      ( update_loc lexbuf name (int_of_string num) true 0;
        token lexbuf
      )
# 1499 "ml/parsing/lexer.ml"

  | 29 ->
# 384 "ml/parsing/lexer.mll"
         ( SHARP )
# 1504 "ml/parsing/lexer.ml"

  | 30 ->
# 385 "ml/parsing/lexer.mll"
         ( AMPERSAND )
# 1509 "ml/parsing/lexer.ml"

  | 31 ->
# 386 "ml/parsing/lexer.mll"
         ( AMPERAMPER )
# 1514 "ml/parsing/lexer.ml"

  | 32 ->
# 387 "ml/parsing/lexer.mll"
         ( BACKQUOTE )
# 1519 "ml/parsing/lexer.ml"

  | 33 ->
# 388 "ml/parsing/lexer.mll"
         ( QUOTE )
# 1524 "ml/parsing/lexer.ml"

  | 34 ->
# 389 "ml/parsing/lexer.mll"
         ( LPAREN )
# 1529 "ml/parsing/lexer.ml"

  | 35 ->
# 390 "ml/parsing/lexer.mll"
         ( RPAREN )
# 1534 "ml/parsing/lexer.ml"

  | 36 ->
# 391 "ml/parsing/lexer.mll"
         ( STAR )
# 1539 "ml/parsing/lexer.ml"

  | 37 ->
# 392 "ml/parsing/lexer.mll"
         ( COMMA )
# 1544 "ml/parsing/lexer.ml"

  | 38 ->
# 393 "ml/parsing/lexer.mll"
         ( MINUSGREATER )
# 1549 "ml/parsing/lexer.ml"

  | 39 ->
# 394 "ml/parsing/lexer.mll"
         ( DOT )
# 1554 "ml/parsing/lexer.ml"

  | 40 ->
# 395 "ml/parsing/lexer.mll"
         ( DOTDOT )
# 1559 "ml/parsing/lexer.ml"

  | 41 ->
# 396 "ml/parsing/lexer.mll"
         ( COLON )
# 1564 "ml/parsing/lexer.ml"

  | 42 ->
# 397 "ml/parsing/lexer.mll"
         ( COLONCOLON )
# 1569 "ml/parsing/lexer.ml"

  | 43 ->
# 398 "ml/parsing/lexer.mll"
         ( COLONEQUAL )
# 1574 "ml/parsing/lexer.ml"

  | 44 ->
# 399 "ml/parsing/lexer.mll"
         ( COLONGREATER )
# 1579 "ml/parsing/lexer.ml"

  | 45 ->
# 400 "ml/parsing/lexer.mll"
         ( SEMI )
# 1584 "ml/parsing/lexer.ml"

  | 46 ->
# 401 "ml/parsing/lexer.mll"
         ( SEMISEMI )
# 1589 "ml/parsing/lexer.ml"

  | 47 ->
# 402 "ml/parsing/lexer.mll"
         ( LESS )
# 1594 "ml/parsing/lexer.ml"

  | 48 ->
# 403 "ml/parsing/lexer.mll"
         ( LESSMINUS )
# 1599 "ml/parsing/lexer.ml"

  | 49 ->
# 404 "ml/parsing/lexer.mll"
         ( EQUAL )
# 1604 "ml/parsing/lexer.ml"

  | 50 ->
# 405 "ml/parsing/lexer.mll"
         ( LBRACKET )
# 1609 "ml/parsing/lexer.ml"

  | 51 ->
# 406 "ml/parsing/lexer.mll"
         ( LBRACKETBAR )
# 1614 "ml/parsing/lexer.ml"

  | 52 ->
# 407 "ml/parsing/lexer.mll"
         ( LBRACKETLESS )
# 1619 "ml/parsing/lexer.ml"

  | 53 ->
# 408 "ml/parsing/lexer.mll"
         ( LBRACKETGREATER )
# 1624 "ml/parsing/lexer.ml"

  | 54 ->
# 409 "ml/parsing/lexer.mll"
         ( RBRACKET )
# 1629 "ml/parsing/lexer.ml"

  | 55 ->
# 410 "ml/parsing/lexer.mll"
         ( LBRACE )
# 1634 "ml/parsing/lexer.ml"

  | 56 ->
# 411 "ml/parsing/lexer.mll"
         ( LBRACELESS )
# 1639 "ml/parsing/lexer.ml"

  | 57 ->
# 412 "ml/parsing/lexer.mll"
         ( BAR )
# 1644 "ml/parsing/lexer.ml"

  | 58 ->
# 413 "ml/parsing/lexer.mll"
         ( BARBAR )
# 1649 "ml/parsing/lexer.ml"

  | 59 ->
# 414 "ml/parsing/lexer.mll"
         ( BARRBRACKET )
# 1654 "ml/parsing/lexer.ml"

  | 60 ->
# 415 "ml/parsing/lexer.mll"
         ( GREATER )
# 1659 "ml/parsing/lexer.ml"

  | 61 ->
# 416 "ml/parsing/lexer.mll"
         ( GREATERRBRACKET )
# 1664 "ml/parsing/lexer.ml"

  | 62 ->
# 417 "ml/parsing/lexer.mll"
         ( RBRACE )
# 1669 "ml/parsing/lexer.ml"

  | 63 ->
# 418 "ml/parsing/lexer.mll"
         ( GREATERRBRACE )
# 1674 "ml/parsing/lexer.ml"

  | 64 ->
# 420 "ml/parsing/lexer.mll"
          ( if !in_annotation then EQUALEQUALGREATER else INFIXOP0 "==>" )
# 1679 "ml/parsing/lexer.ml"

  | 65 ->
# 421 "ml/parsing/lexer.mll"
          ( if !in_annotation then LESSEQUALEQUAL else INFIXOP0 "<==" )
# 1684 "ml/parsing/lexer.ml"

  | 66 ->
# 422 "ml/parsing/lexer.mll"
          ( if !in_annotation then LESSEQUALGREATER else INFIXOP0 "<=>" )
# 1689 "ml/parsing/lexer.ml"

  | 67 ->
# 424 "ml/parsing/lexer.mll"
         ( INFIXOP0 "!=" )
# 1694 "ml/parsing/lexer.ml"

  | 68 ->
# 425 "ml/parsing/lexer.mll"
         ( PLUS )
# 1699 "ml/parsing/lexer.ml"

  | 69 ->
# 426 "ml/parsing/lexer.mll"
         ( MINUS )
# 1704 "ml/parsing/lexer.ml"

  | 70 ->
# 427 "ml/parsing/lexer.mll"
         ( MINUSDOT )
# 1709 "ml/parsing/lexer.ml"

  | 71 ->
# 430 "ml/parsing/lexer.mll"
            ( PREFIXOP(Lexing.lexeme lexbuf) )
# 1714 "ml/parsing/lexer.ml"

  | 72 ->
# 432 "ml/parsing/lexer.mll"
            ( PREFIXOP(Lexing.lexeme lexbuf) )
# 1719 "ml/parsing/lexer.ml"

  | 73 ->
# 434 "ml/parsing/lexer.mll"
            ( INFIXOP0(Lexing.lexeme lexbuf) )
# 1724 "ml/parsing/lexer.ml"

  | 74 ->
# 436 "ml/parsing/lexer.mll"
            ( INFIXOP1(Lexing.lexeme lexbuf) )
# 1729 "ml/parsing/lexer.ml"

  | 75 ->
# 438 "ml/parsing/lexer.mll"
            ( INFIXOP2(Lexing.lexeme lexbuf) )
# 1734 "ml/parsing/lexer.ml"

  | 76 ->
# 440 "ml/parsing/lexer.mll"
            ( INFIXOP4(Lexing.lexeme lexbuf) )
# 1739 "ml/parsing/lexer.ml"

  | 77 ->
# 442 "ml/parsing/lexer.mll"
            ( INFIXOP3(Lexing.lexeme lexbuf) )
# 1744 "ml/parsing/lexer.ml"

  | 78 ->
# 443 "ml/parsing/lexer.mll"
        ( EOF )
# 1749 "ml/parsing/lexer.ml"

  | 79 ->
# 445 "ml/parsing/lexer.mll"
      ( raise (Error(Illegal_character (Lexing.lexeme_char lexbuf 0),
                     Location.curr lexbuf))
      )
# 1756 "ml/parsing/lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf __ocaml_lex_state

and comment lexbuf =
    __ocaml_lex_comment_rec lexbuf 118
and __ocaml_lex_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 451 "ml/parsing/lexer.mll"
      ( comment_start_loc := (Location.curr lexbuf) :: !comment_start_loc;
        comment lexbuf;
      )
# 1769 "ml/parsing/lexer.ml"

  | 1 ->
# 455 "ml/parsing/lexer.mll"
      ( match !comment_start_loc with
        | [] -> assert false
        | [x] -> comment_start_loc := [];
        | _ :: l -> comment_start_loc := l;
                    comment lexbuf;
       )
# 1779 "ml/parsing/lexer.ml"

  | 2 ->
# 462 "ml/parsing/lexer.mll"
      ( reset_string_buffer();
        string_start_loc := Location.curr lexbuf;
        begin try string lexbuf
        with Error (Unterminated_string, _) ->
          match !comment_start_loc with
          | [] -> assert false
          | loc :: _ -> comment_start_loc := [];
                        raise (Error (Unterminated_string_in_comment, loc))
        end;
        reset_string_buffer ();
        comment lexbuf )
# 1794 "ml/parsing/lexer.ml"

  | 3 ->
# 474 "ml/parsing/lexer.mll"
      ( comment lexbuf )
# 1799 "ml/parsing/lexer.ml"

  | 4 ->
# 476 "ml/parsing/lexer.mll"
      ( update_loc lexbuf None 1 false 1;
        comment lexbuf
      )
# 1806 "ml/parsing/lexer.ml"

  | 5 ->
# 480 "ml/parsing/lexer.mll"
      ( comment lexbuf )
# 1811 "ml/parsing/lexer.ml"

  | 6 ->
# 482 "ml/parsing/lexer.mll"
      ( comment lexbuf )
# 1816 "ml/parsing/lexer.ml"

  | 7 ->
# 484 "ml/parsing/lexer.mll"
      ( comment lexbuf )
# 1821 "ml/parsing/lexer.ml"

  | 8 ->
# 486 "ml/parsing/lexer.mll"
      ( comment lexbuf )
# 1826 "ml/parsing/lexer.ml"

  | 9 ->
# 488 "ml/parsing/lexer.mll"
      ( match !comment_start_loc with
        | [] -> assert false
        | loc :: _ -> comment_start_loc := [];
                      raise (Error (Unterminated_comment, loc))
      )
# 1835 "ml/parsing/lexer.ml"

  | 10 ->
# 494 "ml/parsing/lexer.mll"
      ( update_loc lexbuf None 1 false 0;
        comment lexbuf
      )
# 1842 "ml/parsing/lexer.ml"

  | 11 ->
# 498 "ml/parsing/lexer.mll"
      ( comment lexbuf )
# 1847 "ml/parsing/lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec lexbuf __ocaml_lex_state

and string lexbuf =
  lexbuf.Lexing.lex_mem <- Array.create 2 (-1) ;   __ocaml_lex_string_rec lexbuf 144
and __ocaml_lex_string_rec lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 502 "ml/parsing/lexer.mll"
      ( () )
# 1858 "ml/parsing/lexer.ml"

  | 1 ->
let
# 503 "ml/parsing/lexer.mll"
                                  space
# 1864 "ml/parsing/lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 504 "ml/parsing/lexer.mll"
      ( update_loc lexbuf None 1 false (String.length space);
        string lexbuf
      )
# 1870 "ml/parsing/lexer.ml"

  | 2 ->
# 508 "ml/parsing/lexer.mll"
      ( store_string_char(char_for_backslash(Lexing.lexeme_char lexbuf 1));
        string lexbuf )
# 1876 "ml/parsing/lexer.ml"

  | 3 ->
# 511 "ml/parsing/lexer.mll"
      ( store_string_char(char_for_decimal_code lexbuf 1);
         string lexbuf )
# 1882 "ml/parsing/lexer.ml"

  | 4 ->
# 514 "ml/parsing/lexer.mll"
      ( store_string_char(char_for_hexadecimal_code lexbuf 2);
         string lexbuf )
# 1888 "ml/parsing/lexer.ml"

  | 5 ->
# 517 "ml/parsing/lexer.mll"
      ( if in_comment ()
        then string lexbuf
        else begin
(*  Should be an error, but we are very lax.
          raise (Error (Illegal_escape (Lexing.lexeme lexbuf),
                        Location.curr lexbuf))
*)
          let loc = Location.curr lexbuf in
          Location.prerr_warning loc Warnings.Illegal_backslash;
          store_string_char (Lexing.lexeme_char lexbuf 0);
          store_string_char (Lexing.lexeme_char lexbuf 1);
          string lexbuf
        end
      )
# 1906 "ml/parsing/lexer.ml"

  | 6 ->
# 532 "ml/parsing/lexer.mll"
      ( update_loc lexbuf None 1 false 0;
        let s = Lexing.lexeme lexbuf in
        for i = 0 to String.length s - 1 do
          store_string_char s.[i];
        done;
        string lexbuf
      )
# 1917 "ml/parsing/lexer.ml"

  | 7 ->
# 540 "ml/parsing/lexer.mll"
      ( raise (Error (Unterminated_string, !string_start_loc)) )
# 1922 "ml/parsing/lexer.ml"

  | 8 ->
# 542 "ml/parsing/lexer.mll"
      ( store_string_char(Lexing.lexeme_char lexbuf 0);
        string lexbuf )
# 1928 "ml/parsing/lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec lexbuf __ocaml_lex_state

and skip_sharp_bang lexbuf =
    __ocaml_lex_skip_sharp_bang_rec lexbuf 153
and __ocaml_lex_skip_sharp_bang_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 547 "ml/parsing/lexer.mll"
       ( update_loc lexbuf None 3 false 0 )
# 1939 "ml/parsing/lexer.ml"

  | 1 ->
# 549 "ml/parsing/lexer.mll"
       ( update_loc lexbuf None 1 false 0 )
# 1944 "ml/parsing/lexer.ml"

  | 2 ->
# 550 "ml/parsing/lexer.mll"
       ( () )
# 1949 "ml/parsing/lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_skip_sharp_bang_rec lexbuf __ocaml_lex_state

;;

